@@ -1,6 +1,7 @@
 /*******************************************************************
 * reportassistantdialog.cpp
 * Copyright 2009,2010    Dario Andres Rodriguez <andresbajotierra@gmail.com>
+* Copyright 2019 Harald Sitter <sitter@kde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
@@ -31,6 +32,7 @@
 #include "debuggermanager.h"
 #include "backtracegenerator.h"
 
+#include "assistantpage_bugzilla_version.h"
 #include "crashedapplication.h"
 #include "aboutbugreportingdialog.h"
 #include "reportassistantpages_base.h"
@@ -104,6 +106,10 @@ ReportAssistantDialog::ReportAssistantDialog(QWidget * parent) :
     m_conclusionsPage->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
     connect(m_conclusions, &ConclusionPage::finished, this, &ReportAssistantDialog::assistantFinished);
 
+    // Version check page
+    BugzillaVersionPage *versionPage = new BugzillaVersionPage(this);
+    m_pageWidgetMap.insert(QLatin1String(PAGE_BZVERSION_ID), versionPage->item());
+
     //-Bugzilla Login
     BugzillaLoginPage * m_bugzillaLogin =  new BugzillaLoginPage(this);
     connectSignals(m_bugzillaLogin);
@@ -158,6 +164,7 @@ ReportAssistantDialog::ReportAssistantDialog(QWidget * parent) :
     addPage(m_awarenessPage);
     addPage(m_backtracePage);
     addPage(m_conclusionsPage);
+    addPage(versionPage->item());
     addPage(m_bugzillaLoginPage);
     addPage(m_bugzillaDuplicatesPage);
     addPage(m_bugzillaInformationPage);
@@ -174,6 +181,21 @@ ReportAssistantDialog::~ReportAssistantDialog()
 {
 }
 
+void ReportAssistantDialog::setAboutToSend(bool aboutTo)
+{
+    if (aboutTo) {
+        m_nextButtonIconCache = nextButton()->icon();
+        m_nextButtonTextCache = nextButton()->text();
+        nextButton()->setIcon(QIcon::fromTheme(QStringLiteral("document-send")));
+        nextButton()->setText(i18nc("@action button to submit report", "Submit"));
+        return;
+    }
+    nextButton()->setIcon(m_nextButtonIconCache);
+    nextButton()->setText(m_nextButtonTextCache);
+    m_nextButtonIconCache = QIcon();
+    m_nextButtonTextCache = QString();
+}
+
 void ReportAssistantDialog::connectSignals(ReportAssistantPage * page)
 {
     //React to the changes in the assistant pages
@@ -189,28 +211,28 @@ void ReportAssistantDialog::currentPageChanged_slot(KPageWidgetItem * current ,
 
     //Save data of the previous page
     if (before) {
-        ReportAssistantPage* beforePage = dynamic_cast<ReportAssistantPage*>(before->widget());
+        ReportAssistantPage *beforePage = qobject_cast<ReportAssistantPage *>(before->widget());
         beforePage->aboutToHide();
     }
 
     //Load data of the current(new) page
     if (current) {
-        ReportAssistantPage* currentPage = dynamic_cast<ReportAssistantPage*>(current->widget());
+        ReportAssistantPage *currentPage = qobject_cast<ReportAssistantPage *>(current->widget());
         nextButton()->setEnabled(currentPage->isComplete());
         currentPage->aboutToShow();
-    }
 
-    //If the current page is the last one, disable all the buttons until the bug is sent
-    if (current->name() == QLatin1String(PAGE_BZSEND_ID)) {
-        nextButton()->setEnabled(false);
-        backButton()->setEnabled(false);
-        finishButton()->setEnabled(false);
+        // If the current page is the last one, disable all the buttons until the bug is sent
+        if (current->name() == QLatin1String(PAGE_BZSEND_ID)) {
+            nextButton()->setEnabled(false);
+            backButton()->setEnabled(false);
+            finishButton()->setEnabled(false);
+        }
     }
 }
 
 void ReportAssistantDialog::completeChanged(ReportAssistantPage* page, bool isComplete)
 {
-    if (page == dynamic_cast<ReportAssistantPage*>(currentPage()->widget())) {
+    if (page == qobject_cast<ReportAssistantPage *>(currentPage()->widget())) {
         nextButton()->setEnabled(isComplete);
     }
 }
@@ -251,12 +273,17 @@ void ReportAssistantDialog::showHelp()
 //Override KAssistantDialog "next" page implementation
 void ReportAssistantDialog::next()
 {
+    // FIXME: this entire function is a bit weird. It'd likely make more sense to
+    // use the page appropriateness more globally. i.e. mark pages inappropriate
+    // when they are not applicable based on earlier settings done (e.g. put
+    // a conclusion page under/after the awareness page but only mark it
+    // appropriate if the data is not useful. that way kassistantdialog would
+    // just skip over the page).
+
     //Allow the widget to Ask a question to the user before changing the page
-    ReportAssistantPage * page = dynamic_cast<ReportAssistantPage*>(currentPage()->widget());
-    if (page) {
-        if (!page->showNextPage()) {
-            return;
-        }
+    ReportAssistantPage *page = qobject_cast<ReportAssistantPage *>(currentPage()->widget());
+    if (page && !page->showNextPage()) {
+        return;
     }
 
     const QString name = currentPage()->name();
@@ -280,7 +307,11 @@ void ReportAssistantDialog::next()
         if (m_reportInterface->isWorthReporting() &&
             DrKonqi::crashedApplication()->bugReportAddress().isKdeBugzilla())
         {
-            setCurrentPage(m_pageWidgetMap.value(QLatin1String(PAGE_BZLOGIN_ID)));
+            // Depending on whether the page is appropriate either go to version
+            // check page or login page.
+            const auto versionPage = m_pageWidgetMap.value(QLatin1String(PAGE_BZVERSION_ID));
+            const auto loginPage = m_pageWidgetMap.value(QLatin1String(PAGE_BZLOGIN_ID));
+            setCurrentPage(isAppropriate(versionPage) ? versionPage : loginPage);
             return;
         }
     } else if (name == QLatin1String(PAGE_BZDUPLICATES_ID)) {
@@ -360,7 +391,8 @@ void ReportAssistantDialog::closeEvent(QCloseEvent * event)
                 event->accept();
             } else if (ret == KMessageBox::No) {
                 //Save backtrace and accept event (dialog will be closed)
-                DrKonqi::saveReport(reportInterface()->generateReportFullText(false));
+                DrKonqi::saveReport(reportInterface()->generateReportFullText(ReportInterface::DrKonqiStamp::Exclude,
+                                                                              ReportInterface::Backtrace::Complete));
                 event->accept();
             } else {
                 event->ignore();
