@@ -1,6 +1,7 @@
 /*******************************************************************
 * reportassistantdialog.h
 * Copyright 2009    Dario Andres Rodriguez <andresbajotierra@gmail.com>
+* Copyright 2019 Harald Sitter <sitter@kde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
@@ -20,6 +21,7 @@
 #ifndef REPORTASSISTANTDIALOG__H
 #define REPORTASSISTANTDIALOG__H
 
+#include <QIcon>
 #include <QPointer>
 
 #include <KAssistantDialog>
@@ -41,6 +43,12 @@ public:
         return m_reportInterface;
     }
 
+    void setAboutToSend(bool aboutTo);
+
+public Q_SLOTS:
+    void next() override;
+    void back() override;
+
 private Q_SLOTS:
     void currentPageChanged_slot(KPageWidgetItem *, KPageWidgetItem *);
 
@@ -52,9 +60,6 @@ private Q_SLOTS:
 
     void showHelp();
 
-    void next() override;
-    void back() override;
-
     //Override default reject method
     void reject() override;
 
@@ -68,6 +73,9 @@ private:
     ReportInterface *                m_reportInterface = nullptr;
 
     bool                        m_canClose;
+
+    QIcon m_nextButtonIconCache;
+    QString m_nextButtonTextCache;
 };
 
 #endif
