@@ -16,8 +16,8 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 #include "fakebacktracegenerator.h"
-#include <QtCore/QFile>
-#include <QtCore/QTextStream>
+#include <QFile>
+#include <QTextStream>
 
 void FakeBacktraceGenerator::sendData(const QString & filename)
 {
@@ -27,7 +27,7 @@ void FakeBacktraceGenerator::sendData(const QString & filename)
 
     emit starting();
     while (!stream.atEnd()) {
-        emit newLine(stream.readLine() + '\n');
+        emit newLine(stream.readLine() + QLatin1Char('\n'));
     }
     emit newLine(QString());
 }
